/* optrackerobject.h
 *
 * Header file for opcode tracker addition to Python
 *
 * modified January 1997 for Python 1.4
 *
 * Copyright 1996,1997, by Perry A. Stoll <pas@xis.xerox.com>
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of the
 * author not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior
 * permission.
 *
 * THERE ARE NO WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _OPTRACKEROBJECT_H_
#define _OPTRACKEROBJECT_H_
#ifdef __cplusplus
extern "C" {
#endif

#include <mytime.h>
    
#define NOOPCODE -1
#define OPTRACK_MAXNAME_SIZE 24 /* space to hold longest opcode name */
#define OPTRACK_LIST_SIZE 134   /* highest possible opcode number */
#define OPTRACK_OBJ_STRING "__optrack_obj__"

typedef struct
{
    long  op_cnt;                /* number of times executed */
    struct timeval op_cum_ticks;   /* cumulative time spent in this opcode */
    /* # times each given opcode was executed before this one  */
    long op_prev[OPTRACK_LIST_SIZE];
}OpCodeTracker_Info;

typedef struct {
    PyObject_HEAD
    struct timeval start;       /* starting time of opcode execution */
    int prev_op;                /* last opcode executed */
    int timing_op;              /* opcode currently being timed */
    OpCodeTracker_Info ops[OPTRACK_LIST_SIZE];
} PyOpTrackerObject;

extern DL_IMPORT(PyTypeObject) PyOpTracker_Type;

/* If non-NULL, store opcode profiling information in this optracker object */
extern DL_IMPORT(PyObject *)   PyOpTracker_Object;

#define PyOpTracker_Check(op) ((op)->ob_type == &PyOpTracker_Type)

/* Consecutive calls to _StartOpTime will correctly end timing if one
   is already being timed. */
extern void PyOpTracker_StartOpTime Py_PROTO((PyOpTrackerObject *op,
                                              int opcode));

extern void PyOpTracker_EndOpTime    Py_PROTO((PyOpTrackerObject *op,
                                               int opcode));
    
#ifdef __cplusplus
}
#endif
#endif  /* _OPTRACKEROBJECT_H_ */
